var _Combobox_indicator;
import { __classPrivateFieldGet, __classPrivateFieldSet, __decorate } from "tslib";
import { Combobox as FASTComboBox } from '@microsoft/fast-components';
import { attr, Observable } from '@microsoft/fast-element';
import { wasClickInsideElement, wasClickOutsideElement } from '../utils';
import { foundationComboboxStyles as styles } from './combobox.styles';
import { foundationComboboxTemplate as template } from './combobox.template';
/**
 * @tagname %%prefix%%-combobox
 */
export class Combobox extends FASTComboBox {
    constructor() {
        super(...arguments);
        _Combobox_indicator.set(this, void 0);
        /** Controls whether to perform a case-sensitive search in the list of options */
        this.caseSensitive = false;
        /** Controls whether to suppress opening the dropdown when the input field is clicked */
        this.ignoreInputClick = false;
    }
    get value() {
        Observable.track(this, 'value');
        return this._ourValue;
    }
    set value(next) {
        var _a, _b, _c;
        const prev = `${this._ourValue}`;
        if (this.$fastController.isConnected && this.options) {
            const searchFunc = this.caseSensitive
                ? (el) => el.text === next
                : (el) => el.text.toLowerCase() === next.toLowerCase();
            const selectedIndex = this.options ? this.options.findIndex(searchFunc) : -1;
            const prevSelectedValue = (_a = this.options[this.selectedIndex]) === null || _a === void 0 ? void 0 : _a.text;
            const nextSelectedValue = (_b = this.options[selectedIndex]) === null || _b === void 0 ? void 0 : _b.text;
            this.selectedIndex =
                prevSelectedValue !== nextSelectedValue ? selectedIndex : this.selectedIndex;
            next = ((_c = this.firstSelectedOption) === null || _c === void 0 ? void 0 : _c.text) || next;
        }
        if (prev !== next) {
            this._ourValue = next;
            super.valueChanged(prev, next);
            Observable.notify(this, 'value');
        }
    }
    clickHandler(e) {
        if (this.disabled)
            return;
        // click outside, always close
        if (wasClickOutsideElement(e, this)) {
            this.open = false;
            return;
        }
        // always open/close on arrow box
        if (wasClickInsideElement(e, this.indicator)) {
            this.open = !this.open;
            if (this.open)
                this.control.focus();
            return;
        }
        // don't open when clicked in input
        if (this.ignoreInputClick && wasClickInsideElement(e, this.control))
            return;
        return super.clickHandler(e);
    }
    keydownHandler(e) {
        var _a, _b, _c;
        const key = e.key;
        if (e.ctrlKey || e.shiftKey)
            return true;
        switch (key) {
            case 'Enter':
                // don't swallow keys if the combobox isn't open
                if (!this.open)
                    return true;
                break;
            case 'Escape':
                // if value is different from current value, restore current value and leave
                if (this.control.value !== this.currentValue) {
                    this['filter'] = '';
                    this.filterOptions();
                    this.control.value = this.currentValue;
                    this.selectedIndex = (_b = (_a = this.options) === null || _a === void 0 ? void 0 : _a.findIndex((el) => el.text === this.currentValue)) !== null && _b !== void 0 ? _b : -1;
                    (_c = this.options) === null || _c === void 0 ? void 0 : _c.forEach((o, i) => (o.selected = i === this.selectedIndex));
                    this.open = false;
                    return;
                }
                // don't swallow keys if the combobox isn't open
                if (!this.open)
                    return true;
                break;
        }
        const result = super.keydownHandler(e);
        if (key !== 'Enter')
            return result; // for Enter key, don't propagate keypress when selecting
    }
    /**
     * Gets the HTML element for the indicator button
     *
     * @protected
     */
    get indicator() {
        if (!__classPrivateFieldGet(this, _Combobox_indicator, "f"))
            __classPrivateFieldSet(this, _Combobox_indicator, this.shadowRoot.querySelector('[part="indicator"]'), "f");
        return __classPrivateFieldGet(this, _Combobox_indicator, "f");
    }
}
_Combobox_indicator = new WeakMap();
__decorate([
    attr({ mode: 'boolean', attribute: 'case-sensitive' })
], Combobox.prototype, "caseSensitive", void 0);
__decorate([
    attr({ mode: 'boolean', attribute: 'ignore-input-click' })
], Combobox.prototype, "ignoreInputClick", void 0);
// explicitly declaring for extensibility
export const foundationComboboxShadowOptions = {
    delegatesFocus: true,
    mode: 'open',
};
export const defaultComboboxConfig = {
    indicator: `
        <svg
            class="combo-box-indicator"
            part="combo-box-indicator"
            viewBox="0 0 12 7"
            xmlns="http://www.w3.org/2000/svg"
        >
            <path
                d="M11.85.65c.2.2.2.5 0 .7L6.4 6.84a.55.55 0 01-.78 0L.14 1.35a.5.5 0 11.71-.7L6 5.8 11.15.65c.2-.2.5-.2.7 0z"
            />
        </svg>
    `,
};
/**
 * The Foundation Combo-box
 *
 * @public
 * @remarks
 * HTML Element: \<foundation-combo-box\>
 */
export const foundationCombobox = Combobox.compose(Object.assign({ baseName: 'combobox', template,
    styles, shadowOptions: foundationComboboxShadowOptions }, defaultComboboxConfig));
